#include <ctype.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "x_text.hpp"
#include "unique.hpp"
#include "packet.hpp"
#include "select.hpp"

enum option_t { GAME, MAP, N_OPTIONS };

char			*value[2] = { "Duke_Nukem_3D   ", setup.map };
void			beep();
void			editValue(int option, int direction);
void			updateValue(int option);
void			hiliteValue(int option);
void			unhiliteValue(int option);

int			setupKnown;

void beep() {
  sound(1500);
  delay(5);
  nosound();
}

void editValue(int option, int direction) {
  int c, l, done = 0;
  switch (option) {
    case GAME:
      if (!direction) {
	setup.game--; if (setup.game < 0) setup.game = N_GAMES - 1;
      } else {
	setup.game++; if (setup.game >= N_GAMES) setup.game = 0;
      }
      sprintf(value[GAME], gameName[setup.game]);
      break;
    case MAP:
      value[MAP][0] = '\0';
      hiliteValue(MAP);
      gotoxy(36, 7);
      _setcursortype(_SOLIDCURSOR);
      while (!done) {
	c = getch();
	beep();
	if (!c) {
	  c = getch();
	} else {
	  switch(c) {
	    case 8:
	      l = strlen(value[MAP]);
	      if (l) {
		value[MAP][l - 1] = '\0';
		hiliteValue(MAP);
	      }
	      break;
	    case 13:
	      done = 1;
	      break;
	    default:
	      if (isprint(c) && strlen(value[MAP]) < 31) {
		l = strlen(value[MAP]);
		value[MAP][l] = c;
		value[MAP][l + 1] = '\0';
		hiliteValue(MAP);
	      }
	      break;
	  }
	}
      }
      _setcursortype(_NOCURSOR);
      break;
  }
}

void updateValue(int option) {
  int x, y;
  gotoxy(36, 6 + option);
  cprintf(value[option]);
  textcolor(SCREEN_FG);
  textbackground(SCREEN_BG);
  x = wherex(); y = wherey();
  cprintf("                ");
  gotoxy(x, y);
}

void hiliteValue(int option) {
  textcolor(WHITE);
  textbackground(BLACK);
  updateValue(option);
}

void unhiliteValue(int option) {
  textcolor(LIGHTGRAY);
  textbackground(BLACK);
  updateValue(option);
}

void selectInit() {
  textcolor(GREEN);
  textbackground(BLACK);
  gotoxy(30, 6); cprintf("Game:");
  gotoxy(28, 7); cprintf("MAP(s):");
  textcolor(RED); textbackground(BLACK);
  gotoxy(12, 19); cprintf("After you have made your selections, press");
  textcolor(WHITE);
  cprintf(" Alt-G ");
  textcolor(RED);
  cprintf("to begin");
  textcolor(RED); textbackground(BLACK);
  gotoxy(31, 21); cprintf("Press");
  textcolor(WHITE);
  cprintf(" Alt-X ");
  textcolor(RED);
  cprintf("to exit");
  for (int option = MAP; option < N_OPTIONS; option++)
    unhiliteValue((option_t)option);
//  gotoxy(1,15); for (int i = 0; i < 8; i++) cprintf("0123456789");
}

void selectOptions() {
  int c, done = 0, abort = 0, cur_option = GAME;
  _setcursortype(_NOCURSOR);
  selectInit();
  hiliteValue(cur_option);
  while (!done) {
    c = getch();
    if (!c) {
      c = getch();
      switch(c) {
	case 34:
	  done = 1;
	  break;
	case 45:
	  done = abort = 1;
	  break;
	case 72:
	  unhiliteValue(cur_option);
	  cur_option--; if (cur_option < 0) cur_option = N_OPTIONS - 1;
	  hiliteValue(cur_option);
	  beep();
	  break;
	case 75:
	  editValue(cur_option, 0);
	  hiliteValue(cur_option);
	  beep();
	  break;
	case 77:
	  editValue(cur_option, 1);
	  hiliteValue(cur_option);
	  beep();
	  break;
	case 80:
	  unhiliteValue(cur_option);
	  cur_option++; if (cur_option == N_OPTIONS) cur_option = GAME;
	  hiliteValue(cur_option);
	  beep();
	  break;
      }
    } else {
      if (c == 13) {
	editValue(cur_option, 1);
	if (cur_option != MAP) hiliteValue(cur_option);
	beep();
      }
    }
  }
  _setcursortype(_NORMALCURSOR);
  window(1,2,80,25);
  textcolor(SCREEN_FG); textbackground(SCREEN_BG);
  clrscr();
  window(1,1,80,25);
  gotoxy(1,3);
  if (abort) {
    printf("Aborted...\n");
    exit(0);
  }
  setupKnown = 1;
}
